/*******************************************************************************\
 * Copyright (c) 1997 by Markus Koelblin
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
\*******************************************************************************/

/*******************************************************************************\
 * rgbtxt.h                                                                    *
 *                                                                             *
 * Implements X11 color database routines (rgb.txt)                            *
 *                                                                             *
 * Author:  Markus Koelblin (Markus.Koelblin@hadiko.de) 1997                   *
\*******************************************************************************/

#ifndef RGBTXT_H_INCLUDED
#define RGBTXT_H_INCLUDED

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

/*
 * Default location of the rgb.txt file.
 * System dependend - Can be set via Imakefile
 */

#ifndef RGBTXT_DB_NAME
#define RGBTXT_DB_NAME   "/usr/lib/X11/rgb.txt"
#endif

#define RGBTXT_LINE_MAX  1024
#define RGBTXT_NAME_MAX    32

enum { RGBTXT_OPEN_ERROR, RGBTXT_ALLOC_ERROR, RGBTXT_PARSE_ERROR, RGBTXT_ERROR_MAX };


typedef struct tagRgbTxtEntry
{
    unsigned short  red;
    unsigned short  green;
    unsigned short  blue;
    char            name[RGBTXT_NAME_MAX];
}
RgbTxtEntry;

typedef struct tagRgbBase
{
    RgbTxtEntry  *entries;
    int           count;
}
RgbBase;

typedef struct tagRgbQuery
{
    RgbTxtEntry  pattern;
    short        tolerance;
    int          *indices;
    int          count;
}
RgbQuery;


/*******************************************************************************/
/* Open and read X color database and return database entries */

extern int   rgbtxt_create_db( RgbBase *base, char* filename );
extern void  rgbtxt_free_db  ( RgbBase* base );
extern void  rgbtxt_error    ( int err_nr, char* msg );


/* Query for matching colors */

extern int   rgbtxt_create_query( RgbBase *base, RgbQuery *query );
extern void  rgbtxt_free_query  ( RgbQuery *query );

/* Convert rgb values to color name */ 

extern int   rgbtxt_match_name( RgbBase        *base,
				char           **name,
				unsigned short red,
				unsigned short green,
				unsigned short blue );

/* Tolerant conversion from rgb values to color name */ 

extern int   rgbtxt_guess_name( RgbBase        *base,
				char           **name,
				unsigned short red,
				unsigned short green,
				unsigned short blue,
				short          tolerance );

/*******************************************************************************/

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
#endif /* RGBTXT_H_INCLUDED */

